;JT4BCN_9851.ASM    G4JNT  April 2010
;Adapted from JT4_BCN_9851

;Single Frequency / message pre-programmed JT4 beacon

    LIST P = 16F628

		INCLUDE "P16F628.INC"
    __config    0x3F62      ;LV Programming off , OSC HS Mode
    __idlocs    0x802A      
    errorlevel 1, -207  ;Suppress msgs. and warnings about labels after col 1.


    #define 	FData     	PORTA, 0       ;DDS Data on port A
    #define 	WClk     	PORTA ,2
    #define 	FQud     	PORTA, 1
    #define 	MR    		PORTA, 3

    #define     PPS         	PORTB, 0
    #define     LEDGrn      	PORTB, 1	;Connections match
    #define     LEDRed      	PORTB, 2	; those in the RDDS
    #define     GPSIN       	PORTB, 3	; module
    #define     GPSOUT      	PORTB, 4
    #define     TESTLINK    	PORTB, 5	;B6/B7 kept for programming


    #define     PllFlag 	Flags, 0   ;x6 PLL multiplier in AD9851
    #define     TestModeFlag    Flags, 1
    #define     GPSOk           Flags, 2
    #define     HexFlag         Flags, 6

;===================
	org   0x2100
FreqData

 include "jt4_9851.inc"

JT4MsgData
 include "jt4symbs.inc"     ;Left Justified JT4 tones, 4 per byte

;=====================================================						   

	org     0
    nop
    movlw   b'00000000'     ;No interupts until passed thro here once
    movwf   INTCON          ;
    goto    startup         ;jump to main code

;------------------ Interrupt routine, get here 4 cycles after TMR overflow --
ints                            ;/4 prescale &  /256 for ovflow int  = 2441Hz
	bcf     INTCON , T0IF   ;
        movwf   Wreg        ;Save W
	swapf   STATUS , W      ; nb.  movwf does not affect any STATUS bits
	movwf   SaveSts         ;Stave STATUS, without affecting it

	incf    IntCountLo
	btfsc   STATUS, Z
	incf    IntCountHi

        movlw   HIGH(d'558')    ;2441/558 = 4.375Hz symbol rate
	subwf   IntCountHi, W
	btfss   STATUS, Z
	goto    OutInterrupt

        movlw   LOW(d'558')
	subwf   IntCountLo, W
	btfss   STATUS, Z
	goto    OutInterrupt
       ;............             Here at 4.375Hz
	clrf    IntCountLo
	clrf    IntCountHi

	btfsc   SymbCount, 0    ;Flash Red LED at half symbol rate
        bcf     LEDRed
        btfss   SymbCount, 0
        bsf     LEDRed

	movf    SymbCount       ;Test for First symbol, in which case
	btfsc   STATUS, Z       ;  don't do any DDS update
	goto    NotFirstPass

	bsf     FQud  		;Update DDS with LAST frequency, so we need
	nop                     ; to have an extra interrupt to allow this
	nop                     ;  BUT not first time into interrupt
	bcf     FQud
NotFirstPass

        movlw   d'207'
	subwf   SymbCount, W    ;So do the testing for a full message here, 
	btfss   STATUS, Z
	goto    NotLastSymbol

	bcf     INTCON, T0IE    ;Inhibit interrupt after final symbol
	goto    OutInterrupt    ;  and don't do anything else
	;...............
NotLastSymbol
        rrf     SymbCount, W
        movwf   JT4Data        ;JT4Data used as a temporary variable here
        rrf     JT4Data, W
        andlw   0x3F            ;W = SymbCount \ 4, Posn. of this data in EE
        addlw   LOW(JT4MsgData)
        call    GetEE       ;each data byte is called 4 times, for each pair
        movwf   JT4Data        ;JT4Data contains 4 items of freq data

        movf    SymbCount, W
        andlw   0x03            ;Get position in the item (LH = first)
        movwf   PosData
        incf    PosData         ;want 1,2,3 or 4 passes through alignment loop
PosDataLoop
        rlf     JT4Data        ;Data items are left justified in each byte
        rlf     JT4Data        ;shift left 2, 4, or 6 times though C
        decfsz  PosData
        goto    PosDataLoop

        rlf     JT4Data, W     ;One final shift to position at LSBs
        andlw   0x03            ; total of 3,5,7, or 9 left shifts
	call    GenToneData
	call	DataTo9850

	incf    SymbCount       ;Increment here, but tested at start allows 
                                ; the DDS update at last+1 interval, ie. =162
OutInterrupt
	swapf   SaveSts , W     ;Restore STATUS
	movwf   STATUS
	swapf   Wreg            ;Restore W
	swapf   Wreg , W        ;Can't use movf as it  affects Z bit

    retfie
;=====================================
startup 
	clrf    PORTA       ;Special setup to disable comparator on 'F628
	movlw   7
	movwf   CMCON
	
	bsf     STATUS,RP0      ;ram page 1
	movlw   b'00000001'     ;Internal input, Prescalar /4  pull-ups
	movwf   OPTION_REG
	movlw   b'00000000'     ;
	movwf   PORTA           ;
	movlw   b'10101001'     ;B0 PPS, B1/2 LEDs, B3 GPSIN, B4 GPSOUT B5 TestLink 
	movwf   PORTB           ;
	bcf     STATUS,RP0      ;ram page 0

	clrf    PORTB
	clrf    PORTA
	clrf    TMR0
	clrf    IntCountHi
	clrf    IntCountLo
	clrf    Flags		;Set PLL flag in here

	bsf	MR

	bsf     LEDRed
	bcf     LEDGrn
	call    LongDelay
	bcf	MR

	bsf	WClk		;Sets serial mode programming
	nop
	nop
	bcf	WClk
	nop
	nop
	bsf	FQud
	nop
	nop
	bcf	FQud

	call    SetUpDDS	;Sets carrier running
	call    InitGPS
	movlw   LOW(CWMsg)
	movwf   CWMsgCnt
	call	SendCWMsg

	bcf     LEDRed
	bsf     LEDGrn
	call    LongDelay
	bsf     LEDRed
	bcf     LEDGrn
	call    InitGPS     ;Do it twice, just in case
	call    LongDelay

	movlw   b'10000000'     ;Enable global int, not timer overflow int yet
	movwf   INTCON

WaitGPSFirstLock            ;
	movlw   LOW(CWMsg)
	movwf   CWMsgCnt
	call	SendCWMsg
	call	SetUpDDS
                           
	btfss   TESTLINK        
	call    TestMode

	call    GetGPSData      
	btfss   GPSOk
	goto    WaitGPSFirstLock


	bcf     LEDRed
;...........................
MainLoop
	btfss   TESTLINK
	call    TestMode

	call    GetGPSData      ;Sets Minutes, Seconds and RxStatus
	btfsc   Minutes, 0      ;Test for the odd minute
	goto    CarryOn
	
	movf    Seconds, W
	sublw   d'00'
	btfss   STATUS, Z
	goto    CarryOn 
                        ;Now at even minute : 00

	call    SetUpDDS    
	call    WaitPPS     ;Start on the even minute boundary + 1s

	clrf    IntCountLo
	clrf    IntCountHi
	clrf    SymbCount    ;Re-Initialise registers

	bsf     INTCON, T0IE    ;Enable the interrupt, starts JT4 sequence

JT4Loop
	btfsc   INTCON, T0IE ;Sit in here waiting for completion   
	goto    JT4Loop

	call    LongDelay
	call    LongDelay

	movlw   LOW(CWMsg)
	movwf   CWMsgCnt
	call    SendCWMsg

	call    SetUpDDS

CarryOn
	movf    Seconds, W
	sublw   CWMSG2
	btfss   STATUS, Z
	goto    MainLoop

	movlw   LOW(CWMsg)
	movwf   CWMsgCnt
	call    SendCWMsg

	call    SetUpDDS

	goto    MainLoop
;.............................
NoTxValue  
    movlw   4
    movwf   Counter     ;Unique flash pattern to show Random value
NoTxLoop                ;  when transmit sequence has NOT been triggered
    bcf     LEDGrn      ; Flashes R+G 
    bsf     LEDRed
    call    CWDelay
    bsf     LEDGrn
    bcf     LEDRed
    call    CWDelay
    bcf     LEDGrn
    call    CWDelay
    call    CWDelay
    call    CWDelay
    call    CWDelay
    call    CWDelay

    decfsz  Counter
    goto    NoTxLoop

    goto    MainLoop
;===========================================
TestMode                    ;TESTLINK Input grounded
    call    SetUpDDS        ;  Stays here until link is removed
    bsf     LEDRed
    bcf     LEDGrn
TestModeLoop
    btfss   TESTLINK
    goto    TestModeLoop


    bcf     LEDRed
    return
;--------------------------------
GenToneData     	;Changed from WSPR beacon.  Now looks-up each frequency from EE
			;  Allows the non linear offsets rquired for the RDDS 
	movwf	A0		;Save tone no temporarily
	rlf	A0
	rlf	A0, W		;Multiply by 4
	andlw	b'00011100'	;Mask to 8 tone freq offsets, to allow future expansion
	movwf	A0

	movlw	LOW(FreqData)	
	addwf	A0, W		
	call	GetEE
	movwf	D3

	movlw	LOW(FreqData) + 1
	addwf	A0, W		
	call	GetEE
	movwf	D2

	movlw	LOW(FreqData) + 2
	addwf	A0, W		
	call	GetEE
	movwf	D1

	movlw	LOW(FreqData) + 3
	addwf	A0, W		
	call	GetEE
	movwf	D0


	return
;------------------------------------------------------------------

WaitPPS                 ;In case PPS is still high
;~~~~~~~~~~~~~~~~~~~~~~~~~~
  if IgnorePPS == 1
	call	LongDelay	;Mustn't allow the signal to start early
	return			;  so delay 1s here

  else  ;  ~~~~~~~~~~~~~~~~

	btfsc   PPS         	; look for a low to high transition
	goto    WaitPPS

WaitRefPPS              	;Now wait for the rising edge itself
	btfss   PPS        
	goto    WaitRefPPS 
	return
  endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;----------------------
GetEE         ;different from ;F84   EE registers all in bank 1 now 
	bsf     STATUS,RP0      ;ram page 1
	movwf   EEADR
	bsf     EECON1 , RD
	movf    EEDATA , W
	bcf     STATUS , RP0    ;ram page 0
	return
;-----------------------
CWKey
	iorlw	0x00		;Test 0 or 1
	movlw	KEYUPTONE
	btfss	STATUS, Z
	movlw	KEYDWNTONE	;Set depending on 0/1 into 
	
	call	GenToneData
	call    DataTo9850
	bsf     FQud      ;+ve edge of FQud sets 9850 internal reg
	nop
	nop
	nop
	bcf     FQud		
	return
;--------------------------

LongDelay           ;Approx 1 second delay
	movlw   d'250'
	movwf   DelCount2
LongDelLoop
	call    Delay1ms
	call    Delay1ms
	call    Delay1ms
	call    Delay1ms
	decfsz  DelCount2
	goto    LongDelLoop
	return
;-------------------------
Delay1ms        ;5N+5 =  1250 clocks, * 2 = 2500 clocks = 1ms
	movlw   d'249'
	movwf   DelCount
Del1msLoop
	nop
	nop
	decfsz  DelCount
	goto    Del1msLoop
	movlw   d'249'
	movwf   DelCount
Del1ms2Loop
	nop
	nop
	decfsz  DelCount
	goto    Del1ms2Loop
	nop
	nop
	nop
	nop
	return
;-------------------------
GetGPSData          ;Monitors GPSIN line for Time Info
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  if GPSDataType ==0	;Oncore / M12 etc.  9600 baud,  starts @@Ea

	incf    SerCounter
	nop
	bsf     LEDGrn      ;Flash Green LED Until header is received.  If LEDRed
	btfss   GPSOk       ; is enabled, this gives a unique flash pattern
	bcf     LEDGrn      ;Flash in antiphase if GPS is not locked up
	call    PushStack
	call    GetGPSByte
	movwf   Stack0      ;String starts    @@Eamdyyhms
                            ;Stack position - A9876543210
	movf    StackA , W      
	sublw   "@"        
	btfss   STATUS , Z
	goto    GetGPSData
	movf    Stack9 , W     
	sublw   "@"
	btfss   STATUS , Z
	goto    GetGPSData
	movf    Stack8 , W      
	sublw   "E"
	btfss   STATUS , Z
	goto    GetGPSData
	movf    Stack7 , W      
	sublw   "a"
	btfss   STATUS , Z
	goto    GetGPSData
				;Having reached here means correct data message
				; has started and HMS data is stored in stack
	movf    Stack1, W  
	movwf   Minutes
	movf    Stack0, W
	movwf   Seconds

	bcf     LEDGrn      ;Green off means time data has been decoded
	btfss   GPSOk
	bsf     LEDGrn      ;Flash in antiphase if GPS is not locked up

GetGPSStatus        ;Now read in rest of stack data until [cr][lf]
	incf    SerCounter      ;  at the end.  BUT, these may occur as part of
	call    PushStack       ;  binary data, so only test after 74 chars
	call    GetGPSByte     ;  have been received (should be 76, but...)
	movwf   Stack0     

	movlw   d'74'
	subwf   SerCounter, W
	btfss   STATUS, C       ;W = SerCounter - W, if +Ve (C=1) count is ready
	goto    GetGPSStatus    ;  to test for [cr][lf]

	movf    Stack0, W
	sublw   0x0A    ;Stack ends with    ...msdsC[cr][lf]
	btfss   STATUS , Z      ;              65432 1   0 
	goto    GetGPSStatus    ;At some future point, could validate 
	movf    Stack1, W       ; with a checksum calculation.
	sublw   0x0D
	btfss   STATUS , Z
	goto    GetGPSStatus
							;Reached here means terminating [cr][lf] is in
	movf    Stack3, W       ;  the right place, so extract status info
	movwf   RxStatus

	bcf     GPSOk
	movlw   b'00001010' ;Mask the GPS status data to give just the
	andwf   RxStatus, W ; bits that show no proper fix then, only
	btfsc   STATUS, Z   ; if these are all zero,  set GPSOk Flag
	bsf     GPSOk       ; WAS b'01001011', now ignores poor geometry flag
                        ;     and bad almanac 
	clrf    SerCounter      ;Ready for next time
	return
  endif
;--------------------------------------
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  if GPSDataType == 1		;NMEA Format data at 4800 baud

	nop
	bsf     LEDGrn      ;Flash Green LED Until header is received.  If LEDRed
	btfss   GPSOk       ; is enabled, this gives a unique flash pattern
	bcf     LEDGrn      ;Flash in antiphase if GPS is not locked up
	call    PushStack
	call    GetGPSByte
	movwf   Stack0		;String starts    $GPRMC,HHMMSS.ss   (decimal seconds optional)
				;Stack position - CBA9876543210
	movf    StackC , W      
	sublw   "$"        
	btfss   STATUS , Z
	goto    GetGPSData
	movf    StackB , W     
	sublw   "G"
	btfss   STATUS , Z
	goto    GetGPSData
	movf    StackA , W      
	sublw   "P"
	btfss   STATUS , Z
	goto    GetGPSData
	movf    Stack9 , W      
	sublw   "R"
	btfss   STATUS , Z
	goto    GetGPSData
	movf    Stack8 , W      
	sublw   "M"
	btfss   STATUS , Z
	goto    GetGPSData
	movf    Stack7 , W      
	sublw   "C"
	btfss   STATUS , Z
	goto    GetGPSData
	movf    Stack6 , W      ;May be overkill checking all these
	sublw   ","		;   but... whatever....
	btfss   STATUS , Z
	goto    GetGPSData

	movf	Stack2, W	;String starts    $GPRMC,HHMMSS.ss   (decimal seconds optional)
	movwf	BCDLo		;Stack position - CBA9876543210
	movf	Stack3, W
	movwf	BCDHi
	call	BCDtoBIN
	movwf	Minutes

	movf	Stack0, W
	movwf	BCDLo
	movf	Stack1, W
	movwf	BCDHi
	call	BCDtoBIN
	movwf	Seconds

	bcf     LEDGrn      	;Green off means time data has been decoded
	btfss   GPSOk		;Flash in antiphase if GPS is not locked up
	bsf     LEDGrn      	

				;Now continue looking for the comma delimiting the status value
NextComma
	nop
	call    PushStack
	call    GetGPSByte	;After arbitrary seconds formatting...
	movwf	Stack0		;                 xxxx,A,    (or V) 
				;Stack position - 6543210
	sublw	","
	btfss	STATUS, Z
	goto	NextComma
	movf	Stack2, W
	sublw	","
	btfss	STATUS, Z
	goto	NextComma	;
	
	movf	Stack1, W
	sublw	"A"
	bcf	GPSOk
	btfsc	STATUS, Z
	bsf	GPSOk

	return
;----------------------------
BCDtoBIN                ;Converts pairs of ASCII BCD characters to
	movf    BCDHi , W   ;binary in W
	andlw   b'00001111' ;mask from ASCII to digit  = [BCDHi]
	movwf   Temp
	bcf     STATUS , C
	rlf     Temp
	rlf     Temp        ;Temp = 4 * [BCDHi]
	movlw   b'00001111'
	andwf   BCDHi , W   ;W = [BCDHi]
	addwf   Temp        ;Temp = 5 * [BCDHi]
	bcf     STATUS , C
	rlf     Temp
	movf    BCDLo , W
	andlw   b'00001111'
	addwf   Temp        ;Temp = 10 * BCDHi + BCDLo
	movf    Temp , W
	return
;-------------------------------
  endif

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;--------------------------------------
GetGPSByte
 if NMEAPol == 1        ; 1  =  RS232 levels (0/-ve = Idle
    btfss   GPSIN       ; 0  =  Logic levels (1 = Idle)
 else
    btfsc   GPSIN
 endif
    goto    GetGPSByte     ;Loop waiting for start bit

    call    NMEAHalfBitDel    ;Centre of start bit
 if NMEAPol == 1
    btfss   GPSIN
 else
    btfsc   GPSIN       ;Check it's still low
 endif
    goto    GetGPSByte

	movlw   8    
	movwf   Counter
    clrf    Temp        ;probably not needed, but good practice

InitByteLoop
    call    NMEAHalfBitDel
    call    NMEAHalfBitDel    ;
	nop
	bcf     STATUS , C      ;start in known state

 if NMEAPol == 1
    btfss   GPSIN
 else
    btfsc   GPSIN
 endif
	bsf     STATUS , C      ;Data into Carry
    rrf     Temp        ;and then into Temp, LSB first
	decfsz  Counter
    goto    InitByteLoop    ;loop is 8 + 2 * NMEAHalfBitDel long
	 
    call    NMEAHalfBitDel    ;#### Don't really need to wait for stop bit! ####
							; Now have approx 50us of stop bit left
    movf    Temp , W    ;to use this data

    return          ;Exit with received byte in W and Temp
;-----------------------------------------


;-----------------------------------------
NMEAHalfBitDel     

  if GPSDataType == 0	;Oncore etc uses 9600 baud / 260 clocks
	movlw   d'24'   ;  so delay 126 clocks here
  else			; for a total of 126 * 2 + 8 = 260 clocks = 104us
	movlw	d'50'	;NMEA uses 4800 baud / 520 clocks
  endif			;  so delay 256 clocks here

	movwf   DelCount
BaudDelLoop
	nop
	nop
	decfsz  DelCount
	goto    BaudDelLoop
	nop
	return
;-------------------------------
PushStack
	movf    StackE , W
	movwf   StackF
	movf    StackD , W  
	movwf   StackE
	movf    StackC , W
	movwf   StackD
	movf    StackB , W
	movwf   StackC
	movf    StackA , W
	movwf   StackB
	movf    Stack9 , W
	movwf   StackA
	movf    Stack8 , W
	movwf   Stack9
	movf    Stack7 , W
	movwf   Stack8
	movf    Stack6 , W
	movwf   Stack7 
	movf    Stack5 , W
	movwf   Stack6
	movf    Stack4 , W
	movwf   Stack5
	movf    Stack3 , W
	movwf   Stack4
	movf    Stack2 , W
	movwf   Stack3
	movf    Stack1 , W
	movwf   Stack2
	movf    Stack0 , W
	movwf   Stack1

	return
;-------------------------------
;----------------------
SendCWMsg           ;Enter with EE address programmed into CWMsgCnt
	movf    CWMsgCnt, W
	call    GetEE       ;Test for terminator at start = no CW message
	addlw   0
	btfsc   STATUS, Z
	return
	movlw   0           ;Lead in gap makes initial CW clearer
	call    CWKey
	call    CWDelay
	call    CWDelay
	call    CWDelay
	call    CWDelay

CWLoop
	movf    CWMsgCnt, W
	call    GetEE
	addlw   0
	btfsc   STATUS, Z
	goto    DoneCWMsg

	call    SendCWChar
	incf    CWMsgCnt
	goto    CWLoop
DoneCWMsg
	call    CWDelay
	call    CWDelay
	call    CWDelay
	call    CWDelay
	return
;--------------------------
SendCWChar      ;Sends a single CW character
	addlw   0xE0     ;Subtract 32 (add 256 - 32) to point to table
	call    CWTable
	movwf   CWChar      ;Doesn't affect STATUS so...
	movf    CWChar      ;  check for invalid returned code here
	btfsc   STATUS , Z      
	goto    NoChar

	movlw   0x07    
	andwf   CWChar , W      ;Extract element count into W
	movwf   CWCharCnt     
	movlw   6       
	subwf   CWCharCnt , W     ;W = Elcount - 6.  If +ve/0 then C=1, > 5 els
	btfss   STATUS , C
	goto    Send5
	
	rrf     CWCharCnt   	;Puts LSB into C as 110 = .  111 = -
	call    SendEle     	;First  . or -  for 6 element characters
	movlw   5
	movwf   CWCharCnt     ;Force count to 5 for rest of the elements
Send5       
	rlf     CWChar      	;Element to be sent into C.
	call    SendEle     	;   count already in Counter
	decfsz  CWCharCnt   
	goto    Send5
NoChar          		;Inter word gap, and invalid codes.
	bcf     LEDRed
	call    CWDelay
	call    CWDelay
	return
;-------------------
SendEle         ;Data in STATUS , C;  0 = dit  1 = dah
	rrf     Stack0      	;Save C bit temporarily on stack
	movlw   1
	call    CWKey
	bsf     LEDRed
	rlf     Stack0      	;Recovery C bit again, to get dot/dash 
	btfss   STATUS , C      ;test for dot or dash
	goto    SendDot
	call    CWDelay
	call    CWDelay
SendDot
	call    CWDelay
	movlw   0
	call    CWKey
	bcf     LEDRed
	call    CWDelay     ;Inter element gap
	return
;-------------------
CWDelay       
	movlw   CWSPEED
	movwf   DelCount2
CWDelLoop
	call    Delay1ms      
	decfsz  DelCount2
	goto    CWDelLoop
	return
;--------------------------
UpCase
    return

;-------------------------
DecodeHex   ;Single ASCII character to binary with error checking
        ;Untrapped characters remaining ...........
        ;at the moment Upper Case letters G-Z plus characters
        ;  : ; < = > ? @ [ \ ] ^ _ `    will be let through.
        ;Chars 0 - $2F and letters "g" up
        ;are detected.

        bsf     HexFlag         ;Flag for valid hex character
        movwf   Temp            ;
        movlw   0x30
        subwf   Temp            ;subtract $30 and leave in Temp
        btfss   STATUS , C      ;if < $30, result -ve so C=0, error, set=0
        bcf     HexFlag
        
        movf    Temp , W        ;Test for residual from above > d54 ie > "g"
        sublw   d'54'           ;W = 54 - W, if < 54  invalid char, -ve, C=0
        btfss   STATUS , C      ;
        bcf     HexFlag

        movf    Temp , W
        andlw   0x1F            ;mask to 5 bits, resolves upper / lower case
        movwf   Temp            ;if 0 - 9 then Temp is correct, else 7 high

        movf    Temp , W        
        sublw   d'9'            ;W = 9 - W
        btfsc   STATUS , C      ;if -ve, C=0, char > 9, so subtract 7
        goto    ConvDone
        movlw   d'7'
        subwf   Temp            ;
ConvDone
        movf    Temp , W        ;
        andlw   0x0F            ;mask out any rubbish left in
        return                  ;flag set if partly valid 
;-------------------------
InitGPS                 ;Initialise (Oncore type) GPS to SERIN mode, RMC on
	movlw   "@"
    call    SendGPSByte
	movlw   "@"
    call    SendGPSByte
	movlw   "E"
    call    SendGPSByte
	movlw   "a"
    call    SendGPSByte
	movlw   0x01        ;Send Position/Status data message every second
    call    SendGPSByte
	movlw   0x25        ;Checksum  "E" XOR "a" XOR 0x01
    call    SendGPSByte
	movlw   0x0D        ;Terminated with cr/lf
    call    SendGPSByte
	movlw   0x0A
    call    SendGPSByte

	return
;-----------------------------------------
SendGPSByte                    
	movwf   Temp
 if NMEAPol == 1
    bsf     GPSOUT
 else
    bcf     GPSOUT             ;start bit
 endif
    call    NMEAHalfBitDel        ;
    call    NMEAHalfBitDel
	movlw   8
	movwf   Counter
SendGPSDatLoop
	rrf     Temp                ;bit into C


 if NMEAPol == 1
	btfsc   STATUS , C
    bcf     GPSOUT
	btfss   STATUS , C
    bsf     GPSOUT             ;could have 2us jitter depending on 1/0
 else
	btfsc   STATUS , C
    bsf     GPSOUT
	btfss   STATUS , C
    bcf     GPSOUT             ;could have 2us jitter depending on 1/0
 endif

    call    NMEAHalfBitDel
    call    NMEAHalfBitDel
	decfsz  Counter
    goto    SendGPSDatLoop         ;loop 8 + BitDelay long

 if NMEAPol == 1
    bcf     GPSOUT
 else
    bsf     GPSOUT
 endif
    call    NMEAHalfBitDel    
    call    NMEAHalfBitDel        ;Two stop bits
    call    NMEAHalfBitDel
    call    NMEAHalfBitDel

	return
;-----------------------------------------
SetUpDDS		;Sets keying / carrier etc freq
	movlw	REFTONE
	call	GenToneData	
	call    DataTo9850
	bsf     FQud      ;+ve edge of FQud sets 9850 internal reg
	nop
	nop
	nop
	bcf     FQud

	return
;-------------------------
DataTo9850			
        movf    D0 , W
        call    Write9850
        movf    D1 , W
        call    Write9850
        movf    D2 , W
        call    Write9850
        movf    D3 , W
        call    Write9850

        movlw	0         ;Phase = 0
        btfsc   PllFlag
        iorlw	1

        call    Write9850
        bcf     WClk
        return
;--------------------------
Write9850
        movwf   Temp
        movlw   8
        movwf   BitCount
Loop9850      
        rrf     Temp
        btfss   STATUS , C
        bcf     FData
        btfsc   STATUS , C
        bsf     FData
        nop
        nop
        bsf     WClk     
        nop
        nop
	bcf     WClk       ;data changes with -ve edge of clock
        decfsz  BitCount         
        goto    Loop9850

        return
;-------------------------
;============================================


  org 0x700
;--------------------------

;---------------------------------------
CWTable       ;CW Characters in compressed form. 
	movwf   Temp
	movlw   HIGH($)
	movwf   PCLATH    
	movf    Temp , W
	addwf   PCL
	
	retlw   b'00000000'  ; [sp] 00
	retlw   b'00000000'  ;  !   00
	retlw   b'10010110'  ;  "   96
	retlw   b'00010101'  ;  #   15
	retlw   b'00000000'  ;  $   00
	retlw   b'00000000'  ;  %   00
	retlw   b'00000000'  ;  &   00
	retlw   b'11110110'  ;  '   F6
	retlw   b'10110101'  ;  (   B5
	retlw   b'01101111'  ;  )   6F
	retlw   b'01000101'  ;  *   45
	retlw   b'00000000'  ;  +   00
	retlw   b'10011111'  ;  ,   9F     Coding for CW characters of up to
	retlw   b'00001111'  ;  -   0F      6 elements.  Stored as :
	retlw   b'10101110'  ;  .   AE     Bits 7 to 3 make first five CW elements
	retlw   b'10010101'  ;  /   95     Reading  Left to Right 
	retlw   b'11111101'  ;  0   FD     Coded as 0 = dit   1 = dah
	retlw   b'01111101'  ;  1   7D     Bts 2-0 are number of elements except for 
	retlw   b'00111101'  ;  2   3D     110 > 1st element is a dit  
	retlw   b'00011101'  ;  3   1D     111 > 1st element is a dah       
	retlw   b'00001101'  ;  4   0D
	retlw   b'00000101'  ;  5   05
	retlw   b'10000101'  ;  6   85
	retlw   b'11000101'  ;  7   C5
	retlw   b'11100101'  ;  8   E5
	retlw   b'11110101'  ;  9   F5
	retlw   b'00000000'  ;  :   00
	retlw   b'00000000'  ;  ;   00
	retlw   b'00000000'  ;  <   00
	retlw   b'10001101'  ;  =   8D
	retlw   b'00000000'  ;  >   00
	retlw   b'01100110'  ;  ?   66
	retlw   b'11010110'  ;  @   D6
	retlw   b'01000010'  ;  A   42
	retlw   b'10000100'  ;  B   84
	retlw   b'10100100'  ;  C   A4
	retlw   b'10000011'  ;  D   83
	retlw   b'00000001'  ;  E   01
	retlw   b'00100100'  ;  F   24
	retlw   b'11000011'  ;  G   C3
	retlw   b'00000100'  ;  H   04
	retlw   b'00000010'  ;  I   02
	retlw   b'01110100'  ;  J   74
	retlw   b'10100011'  ;  K   A3
	retlw   b'01000100'  ;  L   44
	retlw   b'11000010'  ;  M   C2
	retlw   b'10000010'  ;  N   82
	retlw   b'11100011'  ;  O   E3
	retlw   b'01100100'  ;  P   64
	retlw   b'11010100'  ;  Q   D4
	retlw   b'01000011'  ;  R   43
	retlw   b'00000011'  ;  S   03
	retlw   b'10000001'  ;  T   81
	retlw   b'00100011'  ;  U   23
	retlw   b'00010100'  ;  V   14
	retlw   b'01100011'  ;  W   63
	retlw   b'10010100'  ;  X   94
	retlw   b'10110100'  ;  Y   B4
	retlw   b'11000100'  ;  Z   C4


;================================

;--------------------------------
    cblock 0x20
        Temp
        DelCount
        DelCount2
        IntCountLo
        IntCountHi
        Counter
        SymbCount       ;Counts 0 to 207 for each JT4 symbol
        JT4Data        ;Used in getting the freq data from EE
        PosData         ;Position of the 2 data bits in Byte from EE
        SoftCount       ;Used in amplitude ramping
        Amplitude
        BitCount
        Flags
        Wreg
        SaveSts
        Seconds
        Minutes
        ICount
        CommaCount
        CWChar
        CWMsgCnt
        CWCharCnt
        RxChar
        TonePtr
        SerCounter      ;Count of characters on GPS serial data
        RxStatus    ;GPS Receiver status
        D0      ;D data sent to DDS registers
        D1
        D2
        D3
        A0      ;Just a temporary maths storage location
	BCDHi
	BCDLo
        PNLookup
        PNCounter
        CheckDigit

        AddrHold
        MsgCount

        Stack0
        Stack1
        Stack2
        Stack3
        Stack4
        Stack5
        Stack6
        Stack7
        Stack8
        Stack9
        StackA
        StackB
        StackC
        StackD
        StackE
        StackF
    endc   
;==========================================
		end
